==> c360-input-radio-0.1.0.tgz: package/input-radio.spec.md <==
# Input Radio

## Overview

A radio allows the user to choose only one of a predefined set of mutually exclusive options.

## Use Cases

Radios are used on the forms or settings where user is supposed to make a single choice. The singular property of a radio makes it distinct from a checkbox, which allows more than one (or no) item to be selected and for the unselected state to be restored.

## Non-goals

- Composite radios (larger components that are made up of multiple C360 components)
- Complex radios (components that use a radios internally but do not have a form-like look and feel, such as a [Radio Button Group](https://www.lightningdesignsystem.com/components/radio-button-group/#site-main-content) component)
- Customizations made outside of [Styling Hooks][styling-hooks-jump] are not supported.

## Features

- The API design of the C360 Radio is specific to the [HTML Input][input-spec] element and should not be implemented on any other HTML element. All methods and properties pertain to the HTML element as it was designed by the web platform
- The `default` appearance provides the basic structure of a radio button
- Customization beyond Styling Hooks can be achieved through the [CSS Shadow Parts API][css-shadow-parts]

## Prior Art/Examples

- [Salesforce Lightning Design System](https://www.lightningdesignsystem.com/components/radio-group/)
- [Ant Design](https://ant.design/components/radio/)
- [Atlassian](https://atlassian.design/components/radio/)
- [Material Design](https://material.io/components/radio-buttons)
- [FAST](https://explore.fast.design/components/fast-radio)
- [Morning Star](https://designsystem.morningstar.com/components/radio-button/)
- [Polaris](https://polaris.shopify.com/components/forms/radio-button)
- [Carbon](https://www.carbondesignsystem.com/components/radio-button/usage)
- [Open UI](https://open-ui.org/components/radio-button.research)

## API

The radio component should support all attributes, functionality, and accessibility considerations of the [HTML Input Radio State](https://html.spec.whatwg.org/multipage/input.html#radio-button-state-(type=radio)).

### Custom Element

This specification will apply to a C360 subsystem ecosystem:

`c360-input-radio` - This is owned by the Customer 360 (C360) Team.


## Anatomy and Appearance

The following DOM structure and named part attribute on the leaf node as documented is required to use the C360 Radio CSS.

### Diagram

![Example Diagram](images/c360-input-radio.diagram.png)

### DOM Structure

```html
<host>
  <div part="input-radio">
    <input type="radio" id="radio-id" part="radio" />
    <label part="label" for="radio-id">
      <span part="indicator"></span>
      <slot></slot>
    </label>
  </div>
</host>
```

### Slots

| Slot Name   | Description   | Fallback Content |
| ----------- | -------------- | ---------------- |
| default     | Add the label to the radio | `empty` |

### Parts

| Part Name   | Description                                                         |
| ----------- | ------------------------------------------------------------------- |
| `input-radio`     | Main container of the component                                    |
| `radio`     | Controls input of type radio                                   |
| `indicator` | Controls radio's checked/unchecked state                      |
| `label`     | label part controls the label and the indicator                     |


### Properties & Attributes

| Attribute       | Type     | Default Value | Description                                                                                            |
| --------------- | -------- | ------------- | ------------------------------------------------------------------------------------------------------ |
| `checked`       | `bool`   | `false`       | Whether or not this radio is checked by default                                                      |
| `value`         | `string` | ``        | Is a string containing the radio's value |
| `disabled`      | `bool`   | `false`       | Prevents the user from interacting with the control                                                    |
| `required`      | `bool`   | `false`       | Indicates that the control is invalid unless checked                                                   |
| `name`          | `string` | `null`        | Specifies the name of the control for purposes of form submission                                      |
| `id`            | `string` | `null`        | The ID of the input and the value of the label's `for` attribute                                       |
| `error`         | `bool`   | `null`        | Indicates the radio is in an error state                                                               |
| `size`         | `large/medium/small`   | `large`        | Determines the size of the radio                                                     |


### Events

| Event Name   | Type   | Bubbles | Composed | Cancellable | Dispatch Behavior                                                |
| --------------- | ------ | ------- | -------- | ----------- | ---------------------------------------------------------------- |
| `c360-change` | custom | `true`  | `true`   | `false`     | When the element is `:checked` by clicking or using the keyboard |
| `c360-focus`  | custom | `true`  | `true`   | `false`     | When the element receives focus                                  |
| `c360-blur`   | custom | `true`  | `true`   | `false`     | When the element loses focus                                     |


### Styling Hooks

N/A. Currently there is no customization through styling hooks is being offered for the `c360-input-radio`.

## Behavior

### States & Interactions

| State Group     | States       | Initial State | Description                                                                                                                                 |
| --------------- | ------------ | ------------- | ------------------------------------------------------------------------------------------------------------------------------------------- |
| `checked`       | `true/false` | `null`        | When `true`, associates the control with the parent form data and displays the control with a checked appearance |
| `disabled`      | `true/false` | `null`        | When `true`, disables the control, preventing user interaction and displaying the control with a disabled appearance                        |
| `required`      | `true/false` | `null`        | When `true`, applies the required attribute to the radio                                                   |


## Accessibility

### General Usage

The radio component should conform to the [WAI-ARIA authoring practices](https://www.w3.org/TR/2017/WD-wai-aria-practices-1.1-20170628/examples/radio/radio-1/radio-1.html) for radios.

### Keyboard

Keyboard users can operate the radios as follows:

- The radio is eligible for document focus and if a radio button is checked, focus is set on the checked button
- If none of the radio buttons are checked, focus is set on the first radio button in the group
- The radio can be the target of `click` events (when not disabled) either through `mouse click`, `user tap`, or `keyboard space` key press

### Form input

Control is eligible for all native form capabilities including:

- Form submission
- Form validation
- Form state restoration

### Use with Assistive Technology

Implements the [radio role](https://www.w3.org/TR/wai-aria-1.1/#radio) per the [HTML AAM spec](https://www.w3.org/TR/html-aam-1.0/#html-element-role-mappings).

## Internationalization

All slotted content inside the component should adhere to bi-directional languages such as `ltr` (left to right) and `rtl` (right to left).

## Test Plan

- All radios should have visual regression tests to catch any unintended modifications to the visuals of the radios
- All radios should have a test to catch color contrast regressions that, at a minimum, fulfill the requirements of WCAG 2.1
- All radios should have unit tests to ensure behavior and functionality works as intended

[styling-hooks-jump]: #styling-hooks
[input-spec]: https://html.spec.whatwg.org/multipage/input.html#the-input-element
[css-shadow-parts]: https://drafts.csswg.org/css-shadow-parts/


## Open Questions

- Do we need to use `sds-icon` for checked state?
- Should we provide both `click` and `change` events? or just `change`?
- Do we need `error` state?

==> c360-input-radio-0.2.0.tgz: package/input-radio.spec.md <==
# Input Radio

## Overview

A radio allows the user to choose only one of a predefined set of mutually exclusive options.

## Use Cases

Radios are used on the forms or settings where user is supposed to make a single choice. The singular property of a radio makes it distinct from a checkbox, which allows more than one (or no) item to be selected and for the unselected state to be restored.

## Non-goals

- Composite radios (larger components that are made up of multiple C360 components)
- Complex radios (components that use a radios internally but do not have a form-like look and feel, such as a [Radio Button Group](https://www.lightningdesignsystem.com/components/radio-button-group/#site-main-content) component)
- Customizations made outside of [Styling Hooks][styling-hooks-jump] are not supported.

## Features

- The API design of the C360 Radio is specific to the [HTML Input][input-spec] element and should not be implemented on any other HTML element. All methods and properties pertain to the HTML element as it was designed by the web platform
- The `default` appearance provides the basic structure of a radio button
- Customization beyond Styling Hooks can be achieved through the [CSS Shadow Parts API][css-shadow-parts]

## Prior Art/Examples

- [Salesforce Lightning Design System](https://www.lightningdesignsystem.com/components/radio-group/)
- [Ant Design](https://ant.design/components/radio/)
- [Atlassian](https://atlassian.design/components/radio/)
- [Material Design](https://material.io/components/radio-buttons)
- [FAST](https://explore.fast.design/components/fast-radio)
- [Morning Star](https://designsystem.morningstar.com/components/radio-button/)
- [Polaris](https://polaris.shopify.com/components/forms/radio-button)
- [Carbon](https://www.carbondesignsystem.com/components/radio-button/usage)
- [Open UI](https://open-ui.org/components/radio-button.research)

## API

The radio component should support all attributes, functionality, and accessibility considerations of the [HTML Input Radio State](https://html.spec.whatwg.org/multipage/input.html#radio-button-state-(type=radio)).

### Custom Element

This specification will apply to a C360 subsystem ecosystem:

`c360-input-radio` - This is owned by the Customer 360 (C360) Team.


## Anatomy and Appearance

The following DOM structure and named part attribute on the leaf node as documented is required to use the C360 Radio CSS.

### Diagram

![Diagram of Input Radio Anatomy](images/c360-input-radio.diagram.png)

### DOM Structure

```html
<host>
  <div part="input-radio">
    <input type="radio" id="radio-id" part="radio" />
    <label part="label" for="radio-id">
      <span part="indicator"></span>
      <slot></slot>
    </label>
  </div>
</host>
```

### Slots

| Slot Name   | Description   | Fallback Content |
| ----------- | -------------- | ---------------- |
| default     | Add the label to the radio | `empty` |

### Parts

| Part Name   | Description                                                         |
| ----------- | ------------------------------------------------------------------- |
| `input-radio`     | Main container of the component                                    |
| `radio`     | Controls input of type radio                                   |
| `indicator` | Controls radio's checked/unchecked state                      |
| `label`     | label part controls the label and the indicator                     |


### Properties & Attributes

| Attribute       | Type     | Default Value | Description                                                                                            |
| --------------- | -------- | ------------- | ------------------------------------------------------------------------------------------------------ |
| `checked`       | `bool`   | `false`       | Whether or not this radio is checked by default                                                      |
| `value`         | `string` | `null`        | Is a string containing the radio's value |
| `disabled`      | `bool`   | `false`       | Prevents the user from interacting with the control                                                    |
| `required`      | `bool`   | `false`       | Indicates that the control is invalid unless checked                                                   |
| `name`          | `string` | `null`        | Specifies the name of the control for purposes of form submission                                      |
| `id`            | `string` | `null`        | The ID of the input and the value of the label's `for` attribute                                       |
| `error`         | `bool`   | `null`        | Indicates the radio is in an error state                                                               |
| `size`         | `large/medium/small`   | `large`        | Determines the size of the radio                                                     |


### Events

| Event Name   | Type   | Bubbles | Composed | Cancellable | Dispatch Behavior                                                |
| --------------- | ------ | ------- | -------- | ----------- | ---------------------------------------------------------------- |
| `c360-change` | custom | `true`  | `true`   | `false`     | When the element is `:checked` by clicking or using the keyboard |
| `c360-focus`  | custom | `true`  | `true`   | `false`     | When the element receives focus                                  |
| `c360-blur`   | custom | `true`  | `true`   | `false`     | When the element loses focus                                     |


### Styling Hooks

N/A. Currently there is no customization through styling hooks is being offered for the `c360-input-radio`.

## Behavior

### States & Interactions

| State Group     | States       | Initial State | Description                                                                                                                                 |
| --------------- | ------------ | ------------- | ------------------------------------------------------------------------------------------------------------------------------------------- |
| `checked`       | `true/false` | `null`        | When `true`, associates the control with the parent form data and displays the control with a checked appearance |
| `disabled`      | `true/false` | `null`        | When `true`, disables the control, preventing user interaction and displaying the control with a disabled appearance                        |
| `required`      | `true/false` | `null`        | When `true`, applies the required attribute to the radio                                                   |


## Accessibility

### General Usage

The radio component should conform to the [WAI-ARIA authoring practices](https://www.w3.org/TR/2017/WD-wai-aria-practices-1.1-20170628/examples/radio/radio-1/radio-1.html) for radios.

### Keyboard

Keyboard users can operate the radios as follows:

- The radio is eligible for document focus and if a radio button is checked, focus is set on the checked button
- If none of the radio buttons are checked, focus is set on the first radio button in the group
- The radio can be the target of `click` events (when not disabled) either through `mouse click`, `user tap`, or `keyboard space` key press

### Form input

Control is eligible for all native form capabilities including:

- Form submission
- Form validation
- Form state restoration

### Use with Assistive Technology

Implements the [radio role](https://www.w3.org/TR/wai-aria-1.1/#radio) per the [HTML AAM spec](https://www.w3.org/TR/html-aam-1.0/#html-element-role-mappings).

## Internationalization

All slotted content inside the component should adhere to bi-directional languages such as `ltr` (left to right) and `rtl` (right to left).

## Test Plan

- All radios should have visual regression tests to catch any unintended modifications to the visuals of the radios
- All radios should have a test to catch color contrast regressions that, at a minimum, fulfill the requirements of WCAG 2.1
- All radios should have unit tests to ensure behavior and functionality works as intended

[styling-hooks-jump]: #styling-hooks
[input-spec]: https://html.spec.whatwg.org/multipage/input.html#the-input-element
[css-shadow-parts]: https://drafts.csswg.org/css-shadow-parts/


## Open Questions

- Do we need to use `sds-icon` for checked state?
- Should we provide both `click` and `change` events? or just `change`?
- Do we need `error` state?
